part of katana_module;

extension ModuleBuildContextExtensions on BuildContext {
  /// Get the model adapter.
  ModelAdapter? get model {
    return AdapterScope.of(this).modelAdapter;
  }

  /// Get the platform adapter.
  PlatformAdapter? get platform {
    return AdapterScope.of(this).platformAdapter;
  }

  /// Get the module role.
  List<RoleConfig> get roles {
    return RoleScope.of(this).roles;
  }

  /// Get the app module config.
  AppModule? get app {
    return AppScope.of(this).app;
  }
}

extension RoleConfigExtensions on RoleConfig {
  /// Searches for permissions to be included.
  bool containsPermission(String permission) {
    return permissions.contains(permission);
  }
}

extension NullableRoleConfigExtensions on RoleConfig? {
  /// Searches for permissions to be included.
  bool containsPermission(String permission) {
    if (this == null) {
      return false;
    }
    return this!.permissions.contains(permission);
  }
}

extension IconDataExtensions on IconData {
  /// Convert the icon data to [DynamicMap].
  DynamicMap toMap() {
    return <String, dynamic>{
      "code": codePoint,
      if (fontFamily.isNotEmpty) "family": fontFamily,
      if (fontPackage.isNotEmpty) "package": fontPackage,
    };
  }
}

extension ColorExtensions on Color {
  /// Convert the color to [DynamicMap].
  DynamicMap toMap() {
    return <String, dynamic>{
      "code": value,
    };
  }
}

extension TextStyleExtensions on TextStyle {
  /// Convert the text style to [DynamicMap].
  DynamicMap toMap() {
    return <String, dynamic>{
      if (fontSize != null) "size": fontSize,
      if (color != null) "color": color!.value,
      if (fontWeight != null) "weight": fontWeight!.index,
      if (fontStyle != null) "style": fontStyle!.index,
      if (backgroundColor != null) "backgroundColor": backgroundColor!.value,
    };
  }
}

extension AlignmentExtensions on Alignment {
  /// Convert the alignment to [DynamicMap].
  DynamicMap toMap() {
    return <String, dynamic>{
      "x": x,
      "y": y,
    };
  }
}

extension SizeExtensions on Size {
  /// Convert the size to [DynamicMap].
  DynamicMap toMap() {
    return <String, dynamic>{
      "width": width,
      "height": height,
    };
  }
}

extension EdgeInsetsGeometryExtensions on EdgeInsetsGeometry {
  /// Convert the edge insets to [DynamicMap].
  DynamicMap toMap() {
    final edgeInsets = resolve(TextDirection.ltr);
    return <String, dynamic>{
      "left": edgeInsets.left,
      "right": edgeInsets.right,
      "top": edgeInsets.top,
      "bottom": edgeInsets.bottom,
    };
  }
}

extension ModuleDynamicMapExtensions on DynamicMap {
  /// Convert the icon data from [DynamicMap].
  T? toModule<T extends Module>() {
    return Module._fromMap<T>(this);
  }

  /// Convert the icon data from [DynamicMap].
  IconData? toIconData() {
    if (isEmpty || !containsKey("code")) {
      return null;
    }
    return IconData(
      get("code", 0),
      fontFamily: get<String?>("family", null),
      fontPackage: get<String?>("package", null),
    );
  }

  /// Convert the color from [DynamicMap].
  Color? toColor() {
    if (isEmpty || !containsKey("code")) {
      return null;
    }
    return Color(get("code", 0));
  }

  /// Convert the text style from [DynamicMap].
  TextStyle? toTextStyle() {
    if (isEmpty) {
      return null;
    }
    final color = get<int?>("color", null);
    final backgroundColor = get<int?>("backgroundColor", null);
    return TextStyle(
      fontSize: get<double?>("size", null),
      color: color != null ? Color(color) : null,
      backgroundColor: backgroundColor != null ? Color(backgroundColor) : null,
      fontWeight: FontWeight.values
          .firstWhereOrNull((e) => e.index == get<int?>("weight", null)),
      fontStyle: FontStyle.values
          .firstWhereOrNull((e) => e.index == get<int?>("style", null)),
    );
  }

  /// Convert the alignment from [DynamicMap].
  Alignment? toAlignment() {
    if (isEmpty || !containsKey("x") || !containsKey("y")) {
      return null;
    }
    return Alignment(get("x", 0.0), get("y", 0.0));
  }

  /// Convert the size from [DynamicMap].
  Size? toSize() {
    if (isEmpty || !containsKey("width") || !containsKey("height")) {
      return null;
    }
    return Size(get("width", 0.0), get("height", 0.0));
  }

  /// Convert the edge insets from [DynamicMap].
  EdgeInsets? toEdgeInsets() {
    if (isEmpty ||
        !containsKey("left") ||
        !containsKey("right") ||
        !containsKey("top") ||
        !containsKey("bottom")) {
      return null;
    }
    return EdgeInsets.fromLTRB(
      get("left", 0.0),
      get("top", 0.0),
      get("right", 0.0),
      get("bottom", 0.0),
    );
  }

  /// Convert the menu config from [DynamicMap].
  MenuConfig? toMenuConfig() {
    return MenuConfig._fromMap(this);
  }

  /// Convert the permission from [DynamicMap].
  Permission? toPermission() {
    return Permission._fromMap(this);
  }

  /// Convert the role config from [DynamicMap].
  RoleConfig? toRoleConfig() {
    return RoleConfig._fromMap(this);
  }
}
